
#include "configuration.h"
#include "declarations.h"

char flNumStrPrint = 0;

void my_lcdprint(char *s)
{
  char i;
  char len;
  len = strlen(lcd_buffer);
  for (i = len; i < LCD_WIDTH; i++) s[i] = ' ';
  s[LCD_WIDTH] = 0;

  if (IspReg == 111 && s[9] == 'R' && flNumStrPrint == 0) s[9] = 'N';

  lcd.print(s);

  // MQTT code by max506 & limon
#if USE_MQTT_BROKER

  if (flNumStrPrint == 0) {

    snprintf_P(lcd_mqtt_buf1, MQTT_BUFFER_SIZE, fmt_lcd1, s);

    // Убрать завершающие пробелы
    for (i = strlen(lcd_mqtt_buf1) - 1; ; i--) {
      if (lcd_mqtt_buf1[i] == ' ') lcd_mqtt_buf1[i] = '\0';
      else break;
    }
  }
  else if (flNumStrPrint == 1) {
    snprintf_P(lcd_mqtt_buf2, MQTT_BUFFER_SIZE, fmt_lcd2, s);

    // Убрать завершающие пробелы
    for (i = strlen(lcd_mqtt_buf2) - 1; ; i--) {
      if (lcd_mqtt_buf2[i] == ' ') lcd_mqtt_buf2[i] = '\0';
      else break;
    }
  }
#endif // USE_MQTT_BROKER

  flNumStrPrint++;
}

unsigned char hour, minute, second;
int V1, V2, V3;

void formTRZGDistill()
{
    if (TempDeflBegDistil > 0) sprintf_P(lcd_buffer, PSTR("tD=%3i/%3i %4iW"), temps[TEMP_DEFL], TempDeflBegDistil, Power);
    else  
    sprintf_P(lcd_buffer, PSTR("tK=%3i/%3i %4iW"), temps[TEMP_KUB], TempDeflBegDistil, Power);
}
void formTSAErr()
{
  sprintf_P(lcd_buffer, PSTR("%02u:%02u:%02u TSA!"), hour, minute, second);
  my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
  sprintf_P(lcd_buffer, PSTR("tT=%3i/%3i"), temps[TEMP_TSA], MAX_TEMP_TSA);
}
void DisplaySimpleDistill()
{
  if (DispPage == 0)
  {
    if (IspReg == 104)
    {
      V1 = 1;
      V2 = Temp1P;
    }
    if (IspReg == 105)
    {
      V1 = 2;
      V2 = Temp2P;
    }
    if (IspReg == 106)
    {
      V1 = 3;
      V2 = Temp3P;
    }
    if (IspReg != 107) sprintf_P(lcd_buffer, PSTR("%02u:%02u:%02u %1u-Otbor"), hour, minute, second, V1);

    switch (StateMachine)
    {
    case 0: //Не запущено
    case 1: //Нагрев до температуры 70 градусов
      sprintf_P(lcd_buffer, PSTR("%02u:%02u:%02u Razgon"), hour, minute, second, V1);
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      break;
    case 2: //Ожидание закипания (прогреется дефлегматор)
      if (IspReg != 116)sprintf_P(lcd_buffer, PSTR("%02u:%02u:%02u %1u-O Rzg"), hour, minute, second, V1);
      else sprintf_P(lcd_buffer, PSTR("%02u:%02u:%02u Razgon"), hour, minute, second, V1);
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      formTRZGDistill();
      break;
    case 3: //Дистилляция
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      sprintf_P(lcd_buffer, PSTR("tK=%3i/%3i %4iW"), temps[TEMP_KUB], V2, UstPower);
      break;
    case 4: //Ожидание 60 секунд для охлаждения
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      sprintf_P(lcd_buffer, PSTR("tK=%3i Wait=%4i"), temps[TEMP_KUB], SecondsEnd);
      break;
    case 100: //Окончание
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      sprintf_P(lcd_buffer, PSTR("tK=%3i End"), temps[TEMP_KUB]);
      break;
    case 101: //Температура в ТСА превысила предельную
      formTSAErr();
      break;
    }
  }
  else
  {
    sprintf_P(lcd_buffer, PSTR("D2 T=%3i,%3i,%3i"), temps[TEMP_KUB], temps[TEMP_DEFL], temps[TEMP_TSA]);
    my_lcdprint(lcd_buffer);
    sprintf_P(lcd_buffer, PSTR("%4iW,%3uV,%3uA"), FactPower, (uint16_t)MaxVoltsOut, (uint16_t)MaxIOut);
  }
}

void DisplayData()
{
  int tic1;
  static int PrevState = 0;
  static char flDspDop = 0; // Флаг для отображения информации (через раз)
  char j; // Временная переменная для цикла
  flNumStrPrint = 0;
  flDspDop = !flDspDop;

  hour = Seconds / 3600;
  tic1 = Seconds % 3600;
  minute = tic1 / 60;
  second = tic1 % 60;

  if (FlState == 0)
    lcd.setCursor(0, 0);
  else
  {
    // Если предыдущее состояние было 0, то даем задерку в 1/10 секунды, чтобы не слишком часто обновлялся
    // дисплей.
    if (PrevState == 0)  delay(100);

    if (FlState != PrevState)
    {
      lcd.clear();  //очистка дисплея
      PrevState = FlState;
    }

    lcd.setCursor(0, 0);
  }
  switch (FlState)
  {
    case 0:
      if (DispPage < 2) // Первые две страницы отображаются по-разному, остальные всегда одни и те же
      {
        switch (IspReg)
        {
          case 101: // Displaying
            //  Выводим сетевое напряжение по тому же принципу, что и работают дешевые вольтметры, то есть максимальное количество вольт в сети умножаем на 0,707
            //  это напряжение нужно для калибровки нашего измерителя

            if (DispPage == 0)
            {
              sprintf_P(lcd_buffer, PSTR("Disp0  %02u:%02u:%02u"), hour, minute, second);
              my_lcdprint(lcd_buffer);
              sprintf_P(lcd_buffer, PSTR("Zero=%4u U=%3i"), TicZero, (uint16_t)MaxVoltsOut);
            }

            if (DispPage == 1)
            {
              sprintf_P(lcd_buffer, PSTR("Disp1 t KUB=%3i "), temps[TEMP_KUB]);
              my_lcdprint(lcd_buffer);
              sprintf_P(lcd_buffer, PSTR("DEF=%3i, TSA=%3i"), temps[TEMP_DEFL], temps[TEMP_TSA]);
            }
            break;
          case 102: // Термостат
            if (DispPage == 0)
            {
              sprintf_P(lcd_buffer, PSTR("%02u:%02u:%02u TermSt"), hour, minute, second, Delta);
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              sprintf_P(lcd_buffer, PSTR("t=%3i/%3i(%2i) %1i"), temps[TEMP_TERMOSTAT], TempTerm, Delta, StateMachine);
            }
            else
            {
              sprintf_P(lcd_buffer, PSTR("D2 T=%3i,%3i,%3i"), temps[TEMP_KUB], temps[TEMP_DEFL], temps[TEMP_TSA]);
              my_lcdprint(lcd_buffer);
              sprintf_P(lcd_buffer, PSTR("Power=%4u"), UstPower);
            }
            break;
          case 103: // Регулятор мощности
            if (DispPage == 0)
            {
              sprintf_P(lcd_buffer, PSTR("%02u:%02u:%02u RMU=%3u"), hour, minute, second, (uint16_t)MaxVoltsOut);
              my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
              sprintf_P(lcd_buffer, PSTR("FP=%4u/%4uc%3u"), FactPower, UstPowerReg, indexOut);
            }
            else
            {
              sprintf_P(lcd_buffer, PSTR("D1 T=%3i,%3i,%3i"), temps[TEMP_KUB], temps[TEMP_DEFL], temps[TEMP_TSA]);
              my_lcdprint(lcd_buffer);
              sprintf_P(lcd_buffer, PSTR("Up to auto=%4uW"), Power);
             }
            break;
          case 104: // Первый (недробный) отбор
          case 105: // Второй дробный отбор
          case 106: // Третий дробный отбор
          DisplaySimpleDistill();
            break;
          case 217:
            sprintf_P(lcd_buffer, PSTR("ERR Ds18b20!"));
            my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
            sprintf_P(lcd_buffer, PSTR("Num=%i"), (int) NumErrDs18);
            break;
        }
      }
      // Вторая страница отображения универсальна, там показываем Фактическую мощность, напряжение, ток
      // Фактическая мощность (FP) это расчётная мощность с учётом потерь на симисторе. 
      // На 2 странице можно изменить выходное напряжение регулятора мощности
      if (DispPage == 2)
      {
      sprintf_P(lcd_buffer, PSTR("D2 RM%4uW,%4uW"), (int)UstPowerReg, UstPower);
      my_lcdprint(lcd_buffer);
      sprintf_P(lcd_buffer, PSTR("F%4iW,%3uV,%3uA"), FactPower, (uint16_t)MaxVoltsOut, (uint16_t)MaxIOut);
     }
      // На 3 странице показываем состояние регулятора мощности
      if (DispPage == 3)
      {
        sprintf_P(lcd_buffer, PSTR(" Disp3 UstPower"));
        my_lcdprint(lcd_buffer);
        sprintf_P(lcd_buffer, PSTR(" zPS=%3u, %4uW"), (int)zPSOut, UstPower);
      }
      // На 4 можно сменить состояние процесса.
      if (DispPage == 4)
      {
        sprintf_P(lcd_buffer, PSTR("D4 State Machine"));
        my_lcdprint(lcd_buffer);
        sprintf_P(lcd_buffer, PSTR("   State = %i"), (int)StateMachine);
      }

      if (flDspDop && DispDopInfo > 0) // Если отображаем дополнительную информацию, тогда проверяем ее статус
      {
        if (BeepStateProcess > 1)
        {
          if (DispDopInfo == 3 || DispDopInfo == 4)
          {
            if (!(BeepStateProcess & B00000010) && DispDopInfo == 3) my_beep(BEEP_LONG); // Не пищим, если установлена маска в 2 разряде
            if (!(BeepStateProcess & B00000100) && DispDopInfo == 4) my_beep(BEEP_LONG); // Не пищим, если установлена маска в 3 разряде
          }
          else
            my_beep(BEEP_LONG); // Сначала пищим, предупреждая.
        }
        else
          my_beep(BEEP_LONG); // Сначала пищим, предупреждая.

        if (DispDopInfo == 3) sprintf_P(lcd_buffer, PSTR("Low Power! %3iV"), (uint16_t)MaxVoltsOut); //Выводим на экран инфомацию
        if (DispDopInfo == 4) sprintf_P(lcd_buffer, PSTR("Err temp! ds=%2i"), (int)NumErrDs18); //Выводим на экран инфомацию о ошибке чтения датчика.
      }
      lcd.setCursor(0, 1);
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      break;
    case 100:
      sprintf_P(lcd_buffer, PSTR("Settings Menu"), FlState);
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      sprintf_P(lcd_buffer, PSTR("                "));
      lcd.setCursor(0, 1);
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      break;
    case 101:
      sprintf_P(lcd_buffer, PSTR("Monitoring Mode "));
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      sprintf_P(lcd_buffer, PSTR("                "));
      lcd.setCursor(0, 1);
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      break;
    case 102:
      sprintf_P(lcd_buffer, PSTR("Thermostat"));
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      sprintf_P(lcd_buffer, PSTR("                "));
      lcd.setCursor(0, 1);
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      break;
    case 103:
      sprintf_P(lcd_buffer, PSTR("Power Regulator"));
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      sprintf_P(lcd_buffer, PSTR("                "));
      lcd.setCursor(0, 1);
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      break;
    case 104:
      sprintf_P(lcd_buffer, PSTR("IsR = 1 Distil"));
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      sprintf_P(lcd_buffer, PSTR("(Non-fractional)"));
      lcd.setCursor(0, 1);
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      break;
    case 105:
      sprintf_P(lcd_buffer, PSTR("IsR = 2 DR otbor"));
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      sprintf_P(lcd_buffer, PSTR(" Fractional"));
      lcd.setCursor(0, 1);
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      break;
    case 106:
      sprintf_P(lcd_buffer, PSTR("IsR = 3 DR otbor"));
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      sprintf_P(lcd_buffer, PSTR(" Fractional"));
      lcd.setCursor(0, 1);
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      break;

    case 200:
      sprintf_P(lcd_buffer, PSTR("Max_t_Tst=%5i"), TempTerm);
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      break;
    case 201:
      sprintf_P(lcd_buffer, PSTR("TEH Nominal"));
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      sprintf_P(lcd_buffer, PSTR("Power=%5u"), Power);
      lcd.setCursor(0, 1);
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      break;
    case 202:
      sprintf_P(lcd_buffer, PSTR("Current TEH"));
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      sprintf_P(lcd_buffer, PSTR(" Power=%5u"), UstPowerReg);
      lcd.setCursor(0, 1);
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      break;
    case 203:
      sprintf_P(lcd_buffer, PSTR("ParamUSART=%u"), FlToUSART);
       my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
       break;
    case 204:
      sprintf_P(lcd_buffer, PSTR("dtTermostat=%3i"), Delta);
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      sprintf_P(lcd_buffer, PSTR("t=%3i/%3i(%2i) %1i"), temps[TEMP_TERMOSTAT], TempTerm, Delta, StateMachine);                                                      
      break;
    case 205:
      sprintf_P(lcd_buffer, PSTR("Temp 1 Nedrobn"));
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      sprintf_P(lcd_buffer, PSTR("T_end=%3i"), Temp1P);
      lcd.setCursor(0, 1);
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      break;
    case 206:
      sprintf_P(lcd_buffer, PSTR("Temp 2 Drobn"));
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      sprintf_P(lcd_buffer, PSTR("Distill=%3i"), Temp2P);
      lcd.setCursor(0, 1);
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      break;
    case 207:
      sprintf_P(lcd_buffer, PSTR("Temp 3 Drobn"));
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      sprintf_P(lcd_buffer, PSTR("Distill=%3i"), Temp3P);
      lcd.setCursor(0, 1);
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      break;
    case 208:
      sprintf_P(lcd_buffer, PSTR("Corrections for"));
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      sprintf_P(lcd_buffer, PSTR("ds18b20 values"));
      lcd.setCursor(0, 1);
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      break;
    case 209:
      sprintf_P(lcd_buffer, PSTR("PID Paramters"));
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      sprintf_P(lcd_buffer, PSTR("%4i %4i %4i"), (int)PIDTemp[0], (int)PIDTemp[1], (int)PIDTemp[2]);
      lcd.setCursor(0, 1);
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      break;    
    case 210:
      sprintf_P(lcd_buffer, PSTR("Power simple"));
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      sprintf_P(lcd_buffer, PSTR("Distill=%4i"), PowerDistil);
      lcd.setCursor(0, 1);
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      break;
    case 211:
      sprintf_P(lcd_buffer, PSTR("Temp Begin Dist"));
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      sprintf_P(lcd_buffer, PSTR("(+Def -Kub)=%3i"), TempDeflBegDistil);
      lcd.setCursor(0, 1);
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      break;
    case 212:
      sprintf_P(lcd_buffer, PSTR("BeepEndProc=%1u"), BeepEndProcess);
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      break;
    case 213:
      sprintf_P(lcd_buffer, PSTR("BeepStateProc=%1u"), BeepStateProcess);
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      break;
    case 214:
      sprintf_P(lcd_buffer, PSTR("BeepKeyPress=%1u"), BeepKeyPress);
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      break;
    case 215:
      sprintf_P(lcd_buffer, PSTR("PowerTermostat"));
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      sprintf_P(lcd_buffer, PSTR("Power=%4i"), PowerTermostat);
      lcd.setCursor(0, 1);
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      break;
      case 216:
      sprintf_P(lcd_buffer, PSTR("Power correct"));
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      sprintf_P(lcd_buffer, PSTR("ASC712 =%3i"), (int)CorrectASC712);
      lcd.setCursor(0, 1);
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      break;

      case 300:
      sprintf_P(lcd_buffer, PSTR("Temp%1i=%3u"), nPopr, temps[nPopr]);
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      sprintf_P(lcd_buffer, PSTR("Popr=%3i T=%3u"), ds1820_popr[nPopr], (temps[nPopr] + ds1820_popr[nPopr]));
      lcd.setCursor(0, 1);
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      break;
      case 309:
      sprintf_P(lcd_buffer, PSTR("%4i %4i %4i"), (int)PIDTemp[0], (int)PIDTemp[1], (int)PIDTemp[2]);
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      lcd.setCursor(0, 1);
      if (nPopr == 0) sprintf_P(lcd_buffer, PSTR("   P"));
      if (nPopr == 1) sprintf_P(lcd_buffer, PSTR("        I"));
      if (nPopr == 2) sprintf_P(lcd_buffer, PSTR("             D"));
      my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
      break;
     default:
       break;
  }
  if (flNumStrPrint < 2)
  {
    lcd_buffer[0] = 0;
    my_lcdprint(lcd_buffer); //вывод содержимого буфера на LCD
  }
  NeedDisplaying = false;
}
