// Последнее обновление 2018-08-09 by Phisik
// Phisik: Этот файл бы заметно переписан.
// 1. Добавлена асинхронная работа с UART (как для ввода, так и для вывода)
// 2. Ручное копирование из PROGMEM заменено на strncmp_P()
// 3. Добавлены некоторые поля
// В результате изменений стабильная скорость обмена между ESP & MEGA выросла
// минимум до 500000 бод, дальше не тестировал.
// При такой высокой скорости, можно отправлять больше данных каждые 2 секунды

#include "configuration.h"
#include "declarations.h"

#define MQTT_DEBUG 0

#ifndef MQTT_SERIAL_MODE
#define MQTT_SERIAL_MODE     SERIAL_8N1  // попытка как-то поменять режим работы COM порта  
#endif // !MQTT_SERIAL_MODE

#if USE_MQTT_BROKER

unsigned char CurrentStage = 0;     // текущий этап процесса
unsigned long TimeStage = 0;        // Время работы на текущем этапе
unsigned int CntStop = 0;           // Счетчик СТОПОВ

unsigned long SecPrevBuf = 0;       // время (Seconds) предыдущего обновления данных дисплея
unsigned long SecPrevOper = 0;      // время (Seconds) предыдущего обновления оперативных данных
unsigned long SecCmdDisp = 0;       // время (Seconds) прихода команды, приводящей к изменению информации на дисплее

bool FlagRefreshOnline = true;   // признак необходимости обновления данных в online режиме
bool NeedRefresh = false;        // признак необходимости обновления показаний дисплея
bool extGenerator = false;       // Генератор для проверки целостности линии связи автоматика-scada

char lcd_mqtt_buf1[LCD_BUFFER_SIZE];               // первая строка экрана для отправки по MQTT
char lcd_mqtt_buf2[LCD_BUFFER_SIZE];               // вторая строка экрана для отправки по MQTT

#ifndef MQTT_BUFFER_SIZE
#define MQTT_BUFFER_SIZE 50
#endif // !MQTT_BUFFER_SIZE

// ########################################################
// # Описание форматов для передачи на ESP
// ########################################################

// оперативные данные
PROGMEM const char fmt_lcd1[] = "lcd1=%s";                     // первая строка дисплея
PROGMEM const char fmt_lcd2[] = "lcd2=%s";                     // вторая строка дисплея
PROGMEM const char fmt_stage[] = "stage=%d";                   // этапы текущего процесса
PROGMEM const char fmt_t_kub[] = "t_kub=%d";                   // температура в кубе
PROGMEM const char fmt_t_col[] = "t_col=%d";                   // температура в колонне
PROGMEM const char fmt_t_tsa[] = "t_tsa=%d";                   // температура в ТСА
PROGMEM const char fmt_work_time[] = "work_time=%02u:%02u:%02u";   // время текущего процесса
PROGMEM const char fmt_TimeStage[] = "TimeStage=%02u:%02u:%02u";   // время текущего этапа
PROGMEM const char fmt_extGenerator[] = "extGenerator=%d";         // Генератор для проверки целостности линии связи автоматика-scada

// limon: 2018-07-17
PROGMEM const char fmt_StateMachine[] = "StatMachine=%d";        //31 State Machine
PROGMEM const char fmt_CntMinute[] = "CntMinute=%u";             //32 Время работы автоматики, минут
PROGMEM const char fmt_online[] = "online=%u";                   // надо дать знать ESP в каком режиме мы находимся

// Phisik: выбор режима
PROGMEM const char fmt_IspReg[] = "IspReg=%d";                        // Текущий режим
PROGMEM const char fmt_FactPower[] = "FactPower=%d";                  // Фактическая мощность
PROGMEM const char fmt_MaxVoltsOut[] = "MaxVoltsOut=%d";              // Напряжение в сети

// настройки
PROGMEM const char fmt_BeepKeyPress[] = "BeepKeyPress=%u";             // звук кнопок
PROGMEM const char fmt_BeepEndProc[] = "BeepEndProc=%u";               // звук окончания процесса
PROGMEM const char fmt_BeepStateProc[] = "BeepStateProc=%u";           // звук смены этапа
PROGMEM const char fmt_FlToUSART[] = "FlToUSART=%u";               // вывод в UART
PROGMEM const char fmt_PowerTEH[] = "Power=%u";                    // Мощность тена
PROGMEM const char fmt_PowDistil[] = "PowDistil=%u";               // Мощность дистилляции
PROGMEM const char fmt_TDeflBegDistil[] = "TDeflBegDistil=%d";
PROGMEM const char fmt_Tem1P[] = "Tem1P=%d";                       // температура окончания дистилляции 1
PROGMEM const char val_Tem2P[] =  "Tem2P";                         // температура дистилляции 2
PROGMEM const char fmt_iCorrectASC712[] = "iCorrectASC712=%u";          // Признак того, как надо использовать датчик asc712
PROGMEM const char fmt_ds1820popr0[] = "ds1820popr0=%d";             // Поправка для датчика температуры 0
PROGMEM const char fmt_ds1820popr1[] = "ds1820popr1=%d";             // Поправка для датчика температуры 1
PROGMEM const char fmt_ds1820popr2[] = "ds1820popr2=%d";             // Поправка для датчика температуры 2
PROGMEM const char fmt_DDopInfo[] = "DDopInfo=%u";                   // Флаг отображения аварийной информации

// limon: 2018-07-17
PROGMEM const char fmt_UstPowReg[] = "UstPowReg=%u";               // Мощность регулятора мощности
PROGMEM const char fmt_PowerTermostat[] = "PowerTermostat=%u";     // Мощность для термостата
PROGMEM const char fmt_TmpTerm[] = "TmpTerm=%d";                   // Задание t на термостат (уставка)
PROGMEM const char fmt_TmstDelta[] = "TmstDelta=%d";               // Дельта t на термостат
PROGMEM const char fmt_PID_Temp0[] = "PID_Temp0=%d";               // PID пропорциональный коэффициент
PROGMEM const char fmt_PID_Temp1[] = "PID_Temp1=%d";               // PID интегральный коэффициент
PROGMEM const char fmt_PID_Temp2[] = "PID_Temp2=%d";               // PID дифференциальный коэффициент

// ########################################################
// # Название изменяемых значений
// ########################################################
PROGMEM const char val_BeepKeyPress[] = "BeepKeyPress";            // звук кнопок
PROGMEM const char val_BeepEndProc[] = "BeepEndProc";              // звук окончания процесса
PROGMEM const char val_BeepStateProc[] = "BeepStateProc";          // звук смены этапа
PROGMEM const char val_FlToUSART[] = "FlToUSART";                  // вывод в UART
PROGMEM const char val_online[] = "online";                   // настройка обновления дисплея
PROGMEM const char val_save[] = "save";                       // сохранение в EEPROM
PROGMEM const char val_refresh[] = "refresh";                 // считывание значений настроек
PROGMEM const char val_keystrokes[] = "keystrokes";            // нажатие кнопки
PROGMEM const char val_PowerTEH[] = "PowerT";                  // Мощность ТЭНа
PROGMEM const char val_PowDistil[] = "PowDistil";               // Мощность дистилляции
PROGMEM const char val_TDeflBegDistil[] = "TDeflBegDistil";          // температура окончания разгона
PROGMEM const char val_Tem1P[] = "Tem1P";                       // температура дистилляции 1
PROGMEM const char fmt_Tem2P[] = "Tem2P=%d";                   // температура окончания дистилляции 2

PROGMEM const char val_iCorrectASC712[] = "iCorrectASC712";          // Признак того, как надо использовать датчик asc712
PROGMEM const char val_ds1820popr0[] = "ds1820popr0";             // Поправка для датчика температуры 0
PROGMEM const char val_ds1820popr1[] = "ds1820popr1";             // Поправка для датчика температуры 1
PROGMEM const char val_ds1820popr2[] = "ds1820popr2";             // Поправка для датчика температуры 2
PROGMEM const char val_DDopInfo[] = "DDopInfo";                   // Флаг отображения аварийной информации

// limon: 2018-07-17
PROGMEM const char val_UstPowReg[] = "UstPowReg";               // Мощность регулятора мощности
PROGMEM const char val_PowTermostat[] = "PowTermostat";         // Мощность варка зерно для термостата
PROGMEM const char val_TmpTerm[] = "TmpTerm";                   // Задание t на термостат (уставка)
PROGMEM const char val_TmstDelta[] = "TmstDelta";               // Дельта t на термостат
PROGMEM const char val_PID_Temp0[] = "PID_Temp0";               // PID пропорциональный коэффициент
PROGMEM const char val_PID_Temp1[] = "PID_Temp1";               // PID интегральный коэффициент
PROGMEM const char val_PID_Temp2[] = "PID_Temp2";               // PID дифференциальный коэффициент
// Phisik: 2017-07-17
PROGMEM const char val_IspReg[] = "IspReg";                   // Текущий режим

void initMqtt() {
  // инициализация UART для обмена с модулем ESP
  MQTT_SERIAL.begin(MQTT_SERIAL_BAUDRATE, MQTT_SERIAL_MODE);
}

void mqttSerialPrint(char *s) {
  MQTT_SERIAL.println(s);
}
// Отправка данных на ESP через UART
// Phisik: отправка данных теперь полностью асинхронная, мы не задерживаемся в этой функции
//         отправили, и освобождаем очередь. На следующем loop-e отправим следующую порцию
//         Это позволяет отправлять много информации, не блокирую выполнение остальных функций

#define MQTT_STATE_SETTINGS 40
static short nMqttStateMachine = 0;

bool mqttSendStatus() {
  static boolean bSendAllData = false;
  char buf[MQTT_BUFFER_SIZE + 1] = {0};

  // этот блок требуется для возможности расчета времени работы на текущем этапе и количества СТОПОВ
  // если произошла смена этапа
  if (CurrentStage != StateMachine) {
    // сохраним новый этап и время перехода на новый этап
    CurrentStage = StateMachine;  TimeStage = Seconds;
    //  если перешли в СТОП - увеличим счетчик стопов
    if (StateMachine == 5) CntStop++;
  }

  switch (nMqttStateMachine) {
    case 0: // idle
      if (Seconds % 30 == 0 && Seconds != SecPrevOper) {
        // если наступило время отправки и еще не отправляли в эту секунду
        bSendAllData = true;
        nMqttStateMachine = 1;

        // сохраним время отправки
        SecPrevOper = Seconds;
      } else if (
        ( Seconds % 2 == 0 && Seconds != SecPrevBuf && (FlagRefreshOnline || millis() < 60000L) ) ||
        (NeedRefresh && millis() - SecCmdDisp > 500)
      ) {
        // если наступило время отправки и еще не отправляли в эту секунду и разрешено обновление в online режиме
        // ИЛИ требуется обновление по нажатию кнопки и дисплей успел обновиться (задержка 0.5 сек)
        // Phisik: первые 60 секунд мы отправляем постоянно, вне зависимости от флага FlagRefreshOnline
        //         это надо чтобы esp получило начальные данные после загрузки и подключению к WiFi
        bSendAllData = false;
        nMqttStateMachine = 1;

        // сохраним время отправки
        SecPrevBuf = Seconds;
        // сбросим признак обновления
        NeedRefresh = false;
      } else
        break;

    // Отправляем текущие данные
    case 1: // Вывод LCD экрана
      mqttSerialPrint(lcd_mqtt_buf1);
      mqttSerialPrint(lcd_mqtt_buf2);
      break;
    case 2: // Текущий режим
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_IspReg, IspReg);
      break;
    case 3: // State Machine
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_StateMachine, StateMachine);
      break;
    case 4: // Флаг отображения аварийной информации
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_DDopInfo, DispDopInfo);
      break;
    case 5: // температура в кубе
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_t_kub, temps[TEMP_KUB]);
      break;
    case 6: // температура в колонне
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_t_col, temps[TEMP_RK20]);
      break;
    case 7: // температура в TSA
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_t_tsa, temps[TEMP_TSA]);
      break;
    case 8: // время с момента включения системы/начала процесса
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_work_time, hour, minute, second);
      break;
    case 9: // Фактическая мощность
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_FactPower, FactPower);
      break;
    case 10: // Напряжение в сети
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_MaxVoltsOut, (uint16_t)MaxVoltsOut);
      break;
      // Если не надо все данные отправлять, то переходим в idle state
      if (!bSendAllData) nMqttStateMachine = 0;
      break;
    case 11: // этап процесса
      // Phisik: это дубляж fmt_StateMachine, оставил только для совместимости, надо убрать
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_stage,
                 ((StateMachine > -1 && StateMachine < 9) || (StateMachine > 99 && StateMachine < 103)) ? StateMachine : 404);
      break;
    case 12: // время с момента включения системы/начала процесса
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_work_time, hour, minute, second);
      break;
    case 13: // время текущего этапа
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_TimeStage, (int)((Seconds - TimeStage) / 3600), (int)((Seconds - TimeStage) % 3600) / 60, (int)((Seconds - TimeStage) % 3600) % 60);
      break;
    case 14: // Генератор для проверки целостности линии связи автоматика-scada
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_extGenerator, extGenerator = !extGenerator);
      break;
    case 15: // Время работы автоматики, минут
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_CntMinute, hour * 60 + minute);
      break;
    case 16: // Флаг частого обновления
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_online, FlagRefreshOnline);
      nMqttStateMachine = 0;
      break;

    // Настройки, MQTT_STATE_SETTINGS = 40
    case MQTT_STATE_SETTINGS + 0: // звук при нажатии клавиш
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_BeepKeyPress, BeepKeyPress);
      break;
    case MQTT_STATE_SETTINGS + 1: // звук при окончании процесса
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_BeepEndProc, BeepEndProcess);
      break;
    case MQTT_STATE_SETTINGS + 2: // звук при переходе между этапами процесса
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_BeepStateProc, BeepStateProcess);
      break;
    case MQTT_STATE_SETTINGS + 3: // флаг вывода в USART
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_FlToUSART, FlToUSART);
      break;
    case MQTT_STATE_SETTINGS + 4: // мощность установленного ТЭНа
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_PowerTEH, Power);
      break;
    case MQTT_STATE_SETTINGS + 5: // настройка мощности дистилляции
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_PowDistil, PowerDistil);
      break;
    case MQTT_STATE_SETTINGS + 6: // температура окончания дистилляции 1
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_Tem1P, Temp1P);
      break;
    case MQTT_STATE_SETTINGS + 7: // температура окончания режима разгона
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_PowDistil, PowerDistil);
      break;
    case MQTT_STATE_SETTINGS + 8: // Признак того, как надо использовать датчик asc712
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_iCorrectASC712, CorrectASC712);
      break;
    case MQTT_STATE_SETTINGS + 9: // Поправка для датчика температуры 0
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_ds1820popr0, ds1820_popr[0]);
      break;
    case MQTT_STATE_SETTINGS + 10: // Поправка для датчика температуры 1
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_ds1820popr1, ds1820_popr[1]);
      break;
    case MQTT_STATE_SETTINGS + 11: // Поправка для датчика температуры 2
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_ds1820popr2, ds1820_popr[2]);
      break;
    case MQTT_STATE_SETTINGS + 12: // Мощность регулятора мощности
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_UstPowReg, UstPowerReg);
      break;
    case MQTT_STATE_SETTINGS + 13: // Мощность для термостата
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_PowerTermostat);
      break;
    case MQTT_STATE_SETTINGS + 14: // Задание t на термостат (уставка)
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_TmpTerm, TempTerm);
      break;
    case MQTT_STATE_SETTINGS + 15: // Дельта t на термостат
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_TmstDelta, Delta);
      break;
    case MQTT_STATE_SETTINGS + 16:// PID пропорциональный коэффициент
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_PID_Temp0, PIDTemp[0]);
      break;
    case MQTT_STATE_SETTINGS + 17: // PID интегральный коэффициент
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_PID_Temp1, PIDTemp[1]);
      break;
    case MQTT_STATE_SETTINGS + 18: // PID дифференциальный коэффициент
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_PID_Temp2, PIDTemp[2]);
      break;
    case MQTT_STATE_SETTINGS + 19: // температура окончания дистилляции 1
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_Tem1P, Temp1P);
      break;
    case MQTT_STATE_SETTINGS + 20: // температура окончания дистилляции 2
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_Tem2P, Temp2P);
      break;
    case MQTT_STATE_SETTINGS + 21: // температура окончания режима разгона
      snprintf_P(buf, MQTT_BUFFER_SIZE, fmt_TDeflBegDistil, TempDeflBegDistil);
      break;
    default:
      nMqttStateMachine = 0;
  }
  // Отправляем данные в порт
  if (nMqttStateMachine > 1) mqttSerialPrint(buf);

  // Идем отправлять следующий пункт, если в режиме отправки
  if (nMqttStateMachine > 0) nMqttStateMachine++;
}
// Обработка полученных от ESP данных
void processRecievedData(const char* pub_topic, const char* val)
{
  long ret;
  //unsigned int u_ret;

  // звук кнопок
  if (strncmp_P(pub_topic, val_BeepKeyPress, MQTT_BUFFER_SIZE) == 0)
    BeepKeyPress = val[0] == '1';

  // звук окончания процесса
  else if (strncmp_P(pub_topic, val_BeepEndProc, MQTT_BUFFER_SIZE) == 0)
    BeepEndProcess = val[0] == '1';

  // звук смены этапа
  else if (strncmp_P(pub_topic, val_BeepStateProc, MQTT_BUFFER_SIZE) == 0)
    BeepStateProcess = val[0] == '1';

  // вывод в UART
  else if (strncmp_P(pub_topic, val_FlToUSART, MQTT_BUFFER_SIZE) == 0)
    FlToUSART = val[0] == '1';

  // нажатие кнопки дисплея
  else if (strncmp_P(pub_topic, val_keystrokes, MQTT_BUFFER_SIZE) == 0) {   // изменено для уменьшения количества переменных
    cmdGPRS = val[0];
    // сохранить время получения команды
    SecCmdDisp = millis();
    // установить признак необходимости обновления показаний дисплея
    NeedRefresh = true;
  }

  // настройка обновления дисплея
  else if (strncmp_P(pub_topic, val_online, MQTT_BUFFER_SIZE) == 0) {
    FlagRefreshOnline = val[0] == '1';
  }

  // сохранение в EEPROM
  else if (strncmp_P(pub_topic, val_save, MQTT_BUFFER_SIZE) == 0)
    writeEEPROM();

  // считывание значений настроек
  else if (strncmp_P(pub_topic, val_refresh, MQTT_BUFFER_SIZE) == 0)
    nMqttStateMachine = MQTT_STATE_SETTINGS;

  // мощность установленного ТЭНа
  else if (strncmp_P(pub_topic, val_PowerTEH, MQTT_BUFFER_SIZE) == 0) {
    ret = atol(val);
    if (ret > 0)
      Power = ret;
  }
  // настройка мощности дистилляции
  else if (strncmp_P(pub_topic, val_PowDistil, MQTT_BUFFER_SIZE) == 0) {
    ret = atol(val);
    if (ret > 0 && ret <= Power)
      PowerDistil = ret;
  }
  // температура окончания дистилляции 1
  else if (strncmp_P(pub_topic, val_Tem1P, MQTT_BUFFER_SIZE) == 0) {
    ret = atol(val);
    if (ret > 0 && ret < 1250)
      Temp1P = ret;
  }
  // температура окончания разгона
  else if (strncmp_P(pub_topic, val_TDeflBegDistil, MQTT_BUFFER_SIZE) == 0) {
    ret = atol(val);
    if (ret != 0 && abs(ret) < 1250)
      TempDeflBegDistil = ret;
  }
  // Признак того, как надо использовать датчик asc712
  else if (strncmp_P(pub_topic, val_iCorrectASC712, MQTT_BUFFER_SIZE) == 0) {
    ret = atol(val);
    if (ret >= 0 && ret < 3);
    CorrectASC712 = ret;
  }
  // Поправка для датчика температуры 0
  else if (strncmp_P(pub_topic, val_ds1820popr0, MQTT_BUFFER_SIZE) == 0) {
    ret = atol(val);

    ds1820_popr[0] = ret;
  }
  // Поправка для датчика температуры 1
  else if (strncmp_P(pub_topic, val_ds1820popr1, MQTT_BUFFER_SIZE) == 0) {
    ret = atol(val);

    ds1820_popr[1] = ret;
  }
  // Поправка для датчика температуры 2
  else if (strncmp_P(pub_topic, val_ds1820popr2, MQTT_BUFFER_SIZE) == 0) {
    ret = atol(val);

    ds1820_popr[2] = ret;
  }
  // limon: 2018-07-17

  //44 Мощность регулятора мощности
  else if (strncmp_P(pub_topic, val_UstPowReg, MQTT_BUFFER_SIZE) == 0) {
    ret = atol(val);
    if (ret > 0)
      UstPowerReg = ret;
  }
  //45 Мощность термостата
  else if (strncmp_P(pub_topic, val_PowTermostat, MQTT_BUFFER_SIZE) == 0) {
    ret = atol(val);
    if (ret > 0)
      PowerTermostat = ret;
  }
  //46 Задание t на термостат (уставка)
  else if (strncmp_P(pub_topic, val_TmpTerm, MQTT_BUFFER_SIZE) == 0) {
    ret = atol(val);
    if (ret > 0 && ret <= 1250)
      TempTerm = ret;
  }
  //47 Дельта t на термостат
  else if (strncmp_P(pub_topic, val_TmstDelta, MQTT_BUFFER_SIZE) == 0) {
    ret = atol(val);
    if (ret > 0)
      Delta = ret;
  }
  //48 PID пропорциональный коэффициент
  else if (strncmp_P(pub_topic, val_PID_Temp0, MQTT_BUFFER_SIZE) == 0) {
    ret = atol(val);
    PIDTemp[0] = ret;
  }
  //49 PID интегральный коэффициент
  else if (strncmp_P(pub_topic, val_PID_Temp1, MQTT_BUFFER_SIZE) == 0) {
    ret = atol(val);
    PIDTemp[1] = ret;
  }
  //50 PID дифференциальный коэффициент
  else if (strncmp_P(pub_topic, val_PID_Temp2, MQTT_BUFFER_SIZE) == 0) {
    ret = atol(val);
    PIDTemp[2] = ret;
  }

  // Phisik: limon предложил сделать переключатель режимов, вполне удобно получилось
  //57 Текущий режим
  else if (strncmp_P(pub_topic, val_IspReg, MQTT_BUFFER_SIZE) == 0) {
    ret = atol(val);
    switch (ret) {
      case 101: // Мониторинг
      case 102: // Термостат
      case 103: // Регулятор мощности
      case 104: // Первый (недробный) отбор
      case 105: // Второй дробный отбор
      case 106: // Третий дробный отбор

        IspReg = ret;

        // Внимание! Процесс запустится автоматически!
        StateMachine = 1;
        break;
      default:
        // wrong number received, just skip it
        break;
    }

    IspReg = ret;
  }
}
const short maxTopicNameLength = 20;
int     currentIndex = 0;
char    uartBuffer[MQTT_BUFFER_SIZE] = { 0 };

// Получение данных от ESP
void handleMqttSerial() {
#if MQTT_SERIAL_PING_CHECK
  // Phisik: добавим пинг для UART, чтобы можно было перезапустить/переподключить esp и не дергать сам контроллер
  const int pingPeriod = 14444; // выберем число некратное 2 и 30 сек
  static uint32_t lastUartSendPingTime = millis();
  static uint32_t lastUartRcvPingTime = millis();
#endif

  char* value;

  // Phisik: Не будем блокировать исполнение остального кода
  // Прочитаем сколько есть, остальное на следующей итерации
  int n = MQTT_SERIAL.available();

  // Если буфер пустой, оправляем данные на ESP
  if (n < 1)  mqttSendStatus();

  // В противном случае смотрим, что нам пришло от ESP
  while (n-- > 0) {
    switch (uartBuffer[currentIndex] = MQTT_SERIAL.read()) {

      case '\r':
        // skip all unwanted symbols
        break;
      case '\n':
        // replace new line with zero symbol to provide correct work of strchr() function
        uartBuffer[currentIndex] = 0;

#if MQTT_SERIAL_PING_CHECK
        // answer to ping from ESP
        if (strncmp_P(uartBuffer, PSTR("ping"), MQTT_BUFFER_SIZE) == 0) {
#if MQTT_DEBUG
          DEBUG_SERIAL.println("[INFO][MEGA] Ping was received. Sending pong...");
#endif // MQTT_DEBUG          
          lastUartRcvPingTime = millis();
          MQTT_SERIAL.println("pong");
          currentIndex = 0;
          break;
        }
#endif

        if (value = strchr(uartBuffer, '=')) {
          // don't copy name & value to new buffer, just replace '=' with zero symbol, increment pointer and use both parts as usual strings
          const char* topic = uartBuffer;
          *value++ = 0;

#if MQTT_DEBUG
          DEBUG_SERIAL.print("[MQTT][MEGA] Rcvd: ");
          DEBUG_SERIAL.print(topic);
          DEBUG_SERIAL.print(F("="));
          DEBUG_SERIAL.println(value);
#endif

          // check for empty strings
          if (topic[0] > 0 && value[0] > 0) {
            processRecievedData(topic, value);
          } else {
#if MQTT_DEBUG
            DEBUG_SERIAL.println(F("[MQTT][MEGA] No valid topic was found in received message"));
#endif
          }
        } else {
#if MQTT_DEBUG
          DEBUG_SERIAL.println(F("[MQTT][MEGA] No value was found in received message"));
#endif
        }
        currentIndex = 0;
        break;
      default:
        if (++currentIndex > MQTT_BUFFER_SIZE - 2) {
          currentIndex = 0;
        }
    }
  }

#if MQTT_SERIAL_PING_CHECK
  // Phisik: send ping to esp
  if (millis() - lastUartSendPingTime > pingPeriod) {
#if MQTT_DEBUG
    DEBUG_SERIAL.println(F("[INFO][MEGA] Starting ping-pong UART test. Sending ping..."));
#endif // MQTT_DEBUG
    lastUartSendPingTime = millis();
    MQTT_SERIAL.println("ping");
  }

  // Phisik: reinitialize UART if there was no ping in 2 minutes
  if (millis() - lastUartRcvPingTime > 123456L) {
    lastUartRcvPingTime = millis();
    MQTT_SERIAL.begin(MQTT_SERIAL_BAUDRATE, MQTT_SERIAL_MODE);
    DEBUG_SERIAL.println(F("[WARN][MEGA] MQTT UART seems to lost connection. Reinitializing UART..."));
  }
#endif
}
#endif
